//----------------------------------------------------------------------------
// SVG Viewer - Version 1.0
// A library of SVG renderer
// Copyright (C) 2008 Witold Jaworski
// Contact:	witold.jaworski@airplanes3d.net
//			http://airplanes3d.net
// 
// Contains elements from:
//			AGG - a graphic library (Copyright Maxim Shemanarev, mcseem@antigrain.com)
//          http://antigrain.com
// Uses functions from:
//          Expat - a XML parser library (Copyright James Clark)
// 
// SVG Viewer is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// SVG Viewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with SVG Viewer; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
// MA 02110-1301, USA.
//----------------------------------------------------------------------------
#include "svg_exception.h"
#include "svg_req_filter.h"
#include "svg_filter.h"

namespace svg
{
	//Renders the given elmement onto the result buffer, pointed by req
	//Arguments:
	//	elm:	element, which picture will be filtered and applied to the 
	//			result;
	//	req:	actual request to render 
	//	opacity:optional - level of transparency, used to apply filter
	//			effect to the result buffer (the buffer pointed in the
	//			the <req> request)
	void filter::perform(shape &elm, req_render &req, agg::int8u opacity)
	{
		//create the request:
		req_filter freq(*this,elm,req.trans,req.bounds, 
											    req.renb().ren(),extent_ratio()); 

		if (freq.is_valid()) //current filter region inside 
		{					 //actual clip region:						
			on_request(freq); //submit request to the filter primitivies

			agg::rect_i clip(agg::iround(req.bounds.x1),agg::iround(req.bounds.y1),
							 agg::iround(req.bounds.x2),agg::iround(req.bounds.y2));

			//We have to use the result buffer pixels as the premultiplied ones,
			//so we will create and use another pixel format on the same buffer:
			pixfmt	  &pf = req.renb().ren();
			rendering_buffer rb;
			pixfmt_pre pfp(rb);
			pfp.attach(pf, 0, 0, pf.width()-1, pf.height()-1);

			//copy the result to the result buffer:
			copy(pfp,freq.get(""),freq.x(), freq.y(), &clip, opacity); 
		}
	}

	//request handler:
	handle filter::on_request(request &req)
	{
		switch(req.type())
		{
		case request::initialize:
				initialize(req);
				return skip(req);
		case request::filter:
				if (elements() == 0 && is_valid(m_href)) //If you have no children...
				{	//...redirect this request to the referenced "relative": 
					filter * ref = (filter *) req.get_element(m_href, "filter");
					if (ref) ref->on_request(req); //(and maybe he do the same?)
				}
				else //I have my own filter elements!
				{
					element::forward(req); //to MY filter primitives
				}
				return skip(req);
		default: 
			return element::on_request(req);
		}//switch
	}

} // namespace svg
