//----------------------------------------------------------------------------
// SVG Viewer - Version 1.0
// A library of SVG renderer
// Copyright (C) 2008 Witold Jaworski
// Contact:	witold.jaworski@airplanes3d.net
//			http://airplanes3d.net
// 
// Contains elements from:
//			AGG - a graphic library (Copyright Maxim Shemanarev, mcseem@antigrain.com)
//          http://antigrain.com
// Uses functions from:
//          Expat - a XML parser library (Copyright James Clark)
// 
// SVG Viewer is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// SVG Viewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with SVG Viewer; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
// MA 02110-1301, USA.
//----------------------------------------------------------------------------

#ifndef SVG_STOP_INCLUDED
#include "svg_element.h"
#define SVG_STOP_INCLUDED

namespace svg
{
//////////////////////////////////////////////////////////////////////
// stop: the definition of a single element of a color gradient
//		  
//////////////////////////////////////////////////////////////////////
class stop : public element 
{
	public:
		//////////////////////////////////////////////////////////////////////
		// constructor and overriden virtual methods
		//////////////////////////////////////////////////////////////////////

		stop(element *parent, unsigned level): element(parent, level),
		m_offset(-1.0), //negative offest signals that it is not initialized
		m_color(0,0,0),
		m_opacity(1.0)   
		{}

		virtual const char * tag() {return "stop";}; 
														 

		virtual void on_attribute(parser &prs, const char * name, const char *value)
		{
			if (strcmp(name, "stop-color") == 0) 
			{							   
				m_color = prs.to_color(value);				 
			}
			else 
			if (strcmp(name, "stop-opacity") == 0) 
			{							   
				m_opacity = prs.to_double(value);				 
			}
			else 
			if (strcmp(name, "offset") == 0) 
			{						
				bool percentage;
				m_offset = prs.to_length(value,&percentage);
				if (m_offset < 0) m_offset = 0.0;
				if (m_offset > 1.0) m_offset = 1.0;
			}
			else 
			{
				element::on_attribute(prs,name,value);
			}
		}

		virtual void commit(parser &prs) 
		{
			element::commit(prs);
			m_color.opacity(m_opacity);//prepare color for the use
		}



	public:
		//////////////////////////////////////////////////////////////////////
		// Other methods and fields
		//////////////////////////////////////////////////////////////////////
		//fields accessors:
		double offset() {return m_offset;}

		rgba8 color() {return m_color;}

		bool is_valid() {return m_offset >= 0.0;} //uninitialized stops have 
												  //negative offset 
	protected:
		double	m_offset;	//offset of this element (0.0 ... 1.0)
		rgba8	m_color;	//color, assigned to it (without opacity)
		double	m_opacity;	//0.0 ... 1.0. Separate field, because you never
							//know, what will be first: opacity or color
							//in the attributes of the parsed <stop> element

}; //stop

}; //namespace svg
#endif //SVG_STOP_INCLUDED

